#   -*- coding: utf-8 -*-
import re
import json
from string import Template
from pybuilder.core import init
from pybuilder.core import task
from pybuilder.core import depends
from pybuilder.errors import BuildFailedException
from pybuilder.pluginhelper.external_command import ExternalCommandBuilder
from pybuilder.plugins.python.python_plugin_helper import log_report

SUMMARY = Template("""
    Test results:
        Number of issues identified: ${total}
    Code scanned:
        Total lines of code: ${loc}
        Total lines skipped (#nosec): ${nosec}
    Run metrics:
        Total issues (by severity):
            Undefined: ${severity_undefined}
            Low: ${severity_low}
            Medium: ${severity_medium}
            High: ${severity_high}
        Total issues (by confidence):
            Undefined: ${confidence_undefined}
            Low: ${confidence_low}
            Medium: ${confidence_medium}
            High: ${confidence_high}""")


@init
def init_bandit(project):
    """ initialize complexity task properties
    """
    project.plugin_depends_on('bandit')
    project.set_property_if_unset('bandit_break_build', False)
    project.set_property_if_unset('bandit_confidence_level', 'LOW')
    project.set_property_if_unset('bandit_severity_level', 'LOW')
    project.set_property_if_unset('bandit_skip_ids', None)
    project.set_property_if_unset('bandit_include_testsources', False)
    project.set_property_if_unset('bandit_include_scripts', False)


@task('bandit', description='execute bandit security linter')
@depends('prepare')
def bandit(project, logger):
    """ execute bandit security linter
    """
    output_filename = project.expand_path('$dir_reports/bandit.json')
    include_test_sources = project.get_property('bandit_include_testsources')
    include_scripts = project.get_property('bandit_include_scripts')
    command = get_command(project, output_filename)
    logger.debug(f'Executing bandit security linter: \"{command.as_string}\"')
    result = command.run_on_production_source_files(
        logger,
        include_dirs_only=True,
        include_test_sources=True if include_test_sources else False,
        include_scripts=True if include_scripts else False)
    process_result(project, result, logger, output_filename)


def get_command(project, output_filename):
    """ return bandit command
    """
    command = ExternalCommandBuilder('bandit', project)
    command.use_argument('--recursive')
    command.use_argument('--format')
    command.use_argument('json')
    command.use_argument('--output')
    command.use_argument(output_filename)
    command.use_argument(translate_confidence_level(project.get_property('bandit_confidence_level')))
    command.use_argument(translate_severity_level(project.get_property('bandit_severity_level')))
    skips = project.get_property('bandit_skip_ids')
    if skips:
        command.use_argument('--skip')
        command.use_argument(skips)
    return command


def translate_confidence_level(level):
    """ return confidence level
    """
    if level is None or level == 'LOW':
        return '-i'
    if level == 'MEDIUM':
        return '-ii'
    if level == 'HIGH':
        return '-iii'
    raise ValueError(f'{level} is not a valid confidence level')


def translate_severity_level(level):
    """ return severity level
    """
    if level is None or level == 'LOW':
        return '-l'
    if level == 'MEDIUM':
        return '-ll'
    if level == 'HIGH':
        return '-lll'
    raise ValueError(f'{level} is not a valid severity level')


def read_data(filename):
    """ return dictionary read from bandit json file
    """
    with open(filename) as infile:
        return json.load(infile)


def get_summary_lines(data):
    """ return string summary from bandit data
    """
    metrics = data.get('metrics', {}).get('_totals', {})
    # sanitize metrics keys to be valid variables to enable string substitution
    metrics_vars = {
        'total': len(data.get('results', []))
    }
    for key, value in metrics.items():
        metrics_vars[key.lower().replace('.', '_')] = value
    summary = SUMMARY.substitute(metrics_vars)
    # remove leading spaces from string generated from string template
    return [f'{line[4:]}\n' for line in summary.split('\n')][1:]


def process_result(project, result, logger, output_filename):
    """ process result
    """
    verbose = project.get_property('verbose')
    if verbose:
        log_report(logger, 'bandit', get_summary_lines(read_data(output_filename)))

    if result.exit_code:
        message = f'Bandit security linter detected issues, see {output_filename}'
        fail_build = project.get_property('bandit_break_build')
        if fail_build:
            raise BuildFailedException(message)
        else:
            logger.warn(message)
    else:
        logger.info('Bandit security linter found no issues')



#!/usr/bin/env python
#   -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'pybuilder-bandit',
        version = '0.1.2',
        description = 'Pybuilder plugin for bandit security linter',
        long_description = "[![GitHub Workflow Status](https://github.com/soda480/pybuilder-bandit/workflows/build/badge.svg)](https://github.com/soda480/pybuilder-bandit/actions)\n[![Code Coverage](https://codecov.io/gh/soda480/pybuilder-bandit/branch/main/graph/badge.svg)](https://codecov.io/gh/soda480/pybuilder-bandit)\n[![Code Grade](https://www.code-inspector.com/project/19893/status/svg)](https://frontend.code-inspector.com/project/19893/dashboard)\n[![PyPI version](https://badge.fury.io/py/pybuilder-bandit.svg)](https://badge.fury.io/py/pybuilder-bandit)\n\n# pybuilder-bandit #\n\nA pybuilder plugin that analyzes your project for common security issues using `bandit`. Bandit is a security linter for Python code, for more information refer to the [bandit pypi page](https://pypi.org/project/bandit/).\n\nTo add this plugin into your pybuilder project, add the following line near the top of your build.py:\n```python\nuse_plugin('pypi:pybuilder_bandit')\n```\n\n**NOTE** if you are using Pybuilder version `v0.11.x`, then specify the following version of the plugin:\n```python\nuse_plugin('pypi:pybuilder_bandit', '~=0.1.2')\n```\n\n### Pybuilder bandit properties ###\n\nThe pybuilder task `pyb bandit` will use bandit to scan your project to find common security issues, verbose mode will display to the screen any issues found. The following plugin properties are available to further configure the scan.\n\nName | Type | Default Value | Description\n-- | -- | -- | --\nbandit_break_build | bool | False | Fail build if scan detects any issues\nbandit_confidence_level | str | LOW | Report only issues of a given confidence level or higher: LOW, MEDIUM, HIGH\nbandit_severity_level | str | LOW | report only issues of a given severity level or higher: LOW, MEDIUM, HIGH\nbandit_skip_ids | str | None | comma-separated list of test IDs to skip\nbandit_include_testsources | bool | False | include scanning of project test sources\nbandit_include_scripts | bool | False | include scanning of project scripts\n\nThe plugin properties are set using `project.set_property`, the following is an example of how to set the properties:\n\n```Python\nproject.set_property('bandit_break_build', True)\nproject.set_property('bandit_confidence_level', 'LOW')\nproject.set_property('bandit_severity_level', 'MEDIUM')\nproject.set_property('bandit_skip_ids', 'B110,B315')\nproject.set_property('bandit_include_testsources', True)\nproject.set_property('bandit_include_scripts', True)\n```\n\n### Development ###\n\nClone the repository and ensure the latest version of Docker is installed on your development server.\n\nBuild the Docker image:\n```sh\ndocker image build \\\n-t \\\npybbandit:latest .\n```\n\nRun the Docker container:\n```sh\ndocker container run \\\n--rm \\\n-it \\\n-v $PWD:/pybuilder-bandit \\\npybbandit:latest \\\n/bin/sh\n```\n\nExecute the build:\n```sh\npyb -X\n```",
        long_description_content_type = 'text/markdown',
        classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Other Environment',
            'Environment :: Plugins',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.6',
            'Topic :: Software Development :: Build Tools'
        ],
        keywords = '',

        author = 'Emilio Reyes',
        author_email = 'soda480@gmail.com',
        maintainer = '',
        maintainer_email = '',

        license = 'Apache License, Version 2.0',

        url = 'https://github.com/soda480/pybuilder-bandit',
        project_urls = {},

        scripts = ['scripts/script1.py'],
        packages = ['pybuilder_bandit'],
        namespace_packages = [],
        py_modules = [],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = ['bandit'],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        python_requires = '',
        obsoletes = [],
    )
